@@ -690,6 +690,97 @@ sub updateFormObj
   }
 }
 
+=item scalar lookupSelectName(form =>, item =>, value =>)
+
+ Looks up the name from the specified form value for select/multi-select
+ form items.  This allows you to not have to constantly query the
+ database to resolve values to names.
+
+ optional:
+   form - name of form to work with - defaults to 'main'.
+   value - value to lookup.  If not specified, uses the current
+     value of -Value to determine the value to lookup.  If
+     item is a multi-select, then value must be specified otherwise
+     only the first entry in -Value will be looked up.
+
+ required:
+   item - name of the form item to work with.
+
+ returns undef if the specified form doesn't exist or the
+   specified form item is not a select/multi-select item and errors out.
+   Otherwise, returns the name if found or undef if not found.
+
+ example:
+
+   form = 'main', item = 'color', -Value = 'blue'
+
+   data for color =
+   { names => [ "Red", "Blue", "Green" ],
+     values => [ "red", "blue", "green" ] }
+
+   $obj->lookupSelectName(form => 'main', item => 'color')
+
+   would return "Blue".
+
+   $obj->lookupSelectName(item => 'color', value => 'green')
+
+   would return "Green".
+
+   $obj->lookupSelectName(item => 'color', value => 'purple')
+
+   would return 'undef'.
+
+=cut
+sub lookupSelectName
+{
+  my $self = shift;
+  my %args = ( form => 'main', @_ );
+  my $form = $args{form};
+  my $item = $args{item};
+
+  if (length $form == 0)
+  {
+    $self->missing("form");
+    return undef;
+  }
+  if (!exists $self->data->{$form})
+  {
+    $self->invalid("form", $form, "does not exist in data!");
+    return undef;
+  }
+  if (length $item == 0)
+  {
+    $self->missing("item");
+    return undef;
+  }
+  if (!exists $self->data->{$form}->{$item})
+  {
+    $self->invalid("item", $item, "does not exist in form = '$form'!");
+    return undef;
+  }
+  my $itemType = $self->data->{$form}->{$item}->{-Type};
+  if ($itemType !~ /^(multi-)?select$/)
+  {
+    $self->invalid("item", $item, "-Type = '$itemType' is not a select or multi-select!");
+    return undef;
+  }
+
+  # now determine if they gave us a value or if we need to use the value of -Value.
+  my $Value = $self->get($item);
+  my $value = (exists $args{value} ? $args{value} : (ref ($Value) eq "ARRAY" ? $Value->[0] : $Value));
+
+  # now we loop over the options and find the index of the one that matches the specified value.
+  for (my $i=0; $i < scalar @{$self->data->{$form}->{$item}->{-Options}->{values}}; $i++)
+  {
+    if ($self->data->{$form}->{$item}->{-Options}->{values}->[$i] =~ /^($value)$/)
+    {
+      return $self->data->{$form}->{$item}->{-Options}->{names}->[$i];
+    }
+  }
+
+  return undef;
+}
+
 =item string printXML(fullXML bool, indent int)
 
  This routine returns an XML snippet for this object for use in such
@@ -806,12 +897,17 @@ sub printHTML
   If you need to name the form differently than the form being worked
   with, specify name => "newname".
 
-  If template is specified, then the #LANG=x# substitutions will be done
-  on the string you specify, otherwise we use the template associated
-  with the specified form to do the #LANG=x# substitutions on.
-  Regardless, the template that is ultimately used will have the #LANG=x#
-  substitutions done to it.  If you need to do this yourself, see the
-  langSubstitute() method provided by Portal::Base.
+  The resulting html will have #LANG=x# substitutions done to make sure
+  that any strings in the template or -Label's you wanted languified
+  get done.
+
+  The following sections are languified for you:
+
+      body
+      javascript
+
+  If you need to do this yourself, see the langSubstitute() method
+  provided by Portal::Base.
 
 =cut
 sub printForm
@@ -850,9 +946,8 @@ sub printForm
   # take into account the order array.
   $args{order} = (exists $self->{order}->{$formName} ? $self->{order}->{$formName} : []);
 
-  # fixup the template, replacing all #LANG=x# entries with the mapped version of x.
+  # figure out what template to use.
   my $template = (exists $args{template} ? $args{template} : $self->{template}->{$formName});
-  $template = $self->langSubstitute($template);
 
   %return = $self->{formObj}->generate(
     data => $self->{data}->{$formName},
@@ -860,6 +955,10 @@ sub printForm
     method => $self->{method}, profile => $self->{profile}->{$formName},
     javascript => $self->{javascript}->{$formName}, %args, template => $template);
 
+  # fixup the body and javascript output, replacing all #LANG=x# entries with the mapped version of x.
+  $return{body} = $self->langSubstitute($return{body});
+  $return{javascript} = $self->langSubstitute($return{javascript});
+
   if ($self->{formObj}->error)
   {
     $self->error($self->{formObj}->errorMessage());
